#!/usr/bin/pythonw

#
#	a f p _ a u t h e n t i c a t e
#

#	Created April 1, 2006 by Simon Brown
#	(c) Simon Brown, All rights reserved

#	This script was written as a way to provide a user friendly way of validating passwords
#	when very strict password policies are used with email clients, such as Entourage, that
#	do not handle password problems at all well.
#
#	By preflighting the passwords using AFP login we can check for various password situations,
#	and return predictable results. There are some unavoidable anomalies, as on Panther (Mac OS 10.3)
#	clients the OS will throw up alerts when accessing accounts that are locked and when incorrect
#	passwords are used.
#
#	Although this script does display its progress, the intention is that another script will
#	be used to provide the alerts and dialogs before and after calling this script. An AppleScript
#	Studio app can do the job nicely.
# 
#	INPUT
#		afp_authenticate <host> <volume name> <user> <password> [<keep mounted>]
#			1 - host = host name of AFP server
#			2 - volume name = name of volume to mount on host
#			3 - user = account name to check
#			4 - password = password to check
#			5 - keep mounted = optional, T to keep network volume mounted after check.
#
#	Example:
#		afp_authenticate 'emailserver.mycompany.com' 'sharedVolume' 'myaccount' 'mypassword' T
#
#	Error Results:
#		 -1 - Bad or missing parameter(s)
#		  2 - No mount point on call to mount_afp
#		 10 - No network link found during ifconfig test
#		 11 - Device Busy, couldn't unmount volume
#		 12 = mount_afp says volume already mounted
#		 32 - Change password (Tiger only)
#		 75 - Change password (Panther only)
#		 97 - Wrong password (Panther displays alert)
#		120 - Account is locked (Panther displays alert)
#		211 - Network problem during mount

#	060401	Created file.
#			Able to unmount, mount, and then unmount again
#			Added progress bar to show status
#			Added flag to retain mount of volume
#	060402	Now accept host, user, password, and keep mounted flag as command line options
#	060404	Updated comments
#			Removed extraneous sleep after call to mountVolume/unmountVolume
#			Now use global to set pause length after error.
#			

import EasyDialogs

# Put up a progress bar while we wait for things to happen.
gProgBar = EasyDialogs.ProgressBar ("AFP Authenticate",6,"Initializing")

# How many seconds to wait after an error
gWaitAfterErr = 1

import commands
import os
import string
import sys
import time

#
#	V a r i a b l e s
#

try:
	hostName = sys.argv[1]				# The AFP file server host name
	volName = sys.argv[2]				# The volume name that we will be testing against
	volPath = "/Volumes/" + volName		# Path to the mount point.
	volPathQouted = "'" + volPath + "'"	# Qouted path to mount point when using shell commands
	usrName = sys.argv[3]				# Account name to check
	passwd = sys.argv[4]				# Account password to use

except:
	gProgBar.label("Error! Bad or missing parameters")
	gProgBar.inc(1)
	time.sleep(gWaitAfterErr)
	sys.exit (-1)

keepMounted = False						# The default choice is to unmount the volume when done.
if len (sys.argv) == 6:					# Was there a 5th, optional, parameter.
	keepMounted = (sys.argv[5] == 'T')	# Will set to True if T is given, else returns false.
		
#
#	c h e c k N e t w o r k
#

def checkNetwork():
	global gProgBar

	gProgBar.label("Checking network connection")
	gProgBar.inc(1)
	
	# Get info. on any interfaces currently in use.
	netStatus = commands.getoutput ('ifconfig -u')
	
	# Any of the interfaces actually active (has link)?
	if netStatus.find ('status: active') == -1:
		gProgBar.label("Error! No active network connection")
		gProgBar.inc(1)
		time.sleep(gWaitAfterErr)
		sys.exit (10)

#
#	c r e a t e M o u n t
#

def createMount():

	gProgBar.label("Creating mount point ")
	gProgBar.inc(1)

	try:
		# Attempt to create the mount point.
		os.mkdir (volPath)

	except:
		gProgBar.label("Warning: Problem creating mount point")
		gProgBar.inc(1)
		time.sleep(gWaitAfterErr)

#
#	m o u n t V o l u m e
#

def mountVolume():
	global gProgBar
	
	gProgBar.label ('Mounting network volume')
	gProgBar.inc(1)
	
	mountStr=commands.getoutput ("mount_afp 'afp://" + usrName + ":" + passwd+ "@" + hostName + "/" + volName + "' " + volPathQouted)
#	print mountStr

	# On Panther, mount_afp's result is always set to 255 if there's an error.	
	# Instead of using the result, we'll parse out the last word in output, which is the error code we want
	# on both Panther and Tiger.
	errStr = mountStr.split(' ')[-1]

	# Is the volume still mounted? We don't have an error # then.
	if errStr == 'mounted':
		gProgBar.label("Error! Network volume already mounted")
		gProgBar.inc(1)
		time.sleep(1)
		errNo = 12
		
	else:
		errNo = int(errStr)
		
		# This isn't actually an error #, but a succesful mount returns 0020
		# as the last word in its output.
		if errNo == 20:
			gProgBar.label("Authentication succesful")
			gProgBar.inc(3)
			errNo = 0
			
		elif errNo == 2:
			gProgBar.label("Error! Mount point not found")
			gProgBar.inc(gWaitAfterErr)
			time.sleep(1)

		elif errNo == -1069:
			gProgBar.label("Error! Unable to connect to server")
			gProgBar.inc(1)
			time.sleep(gWaitAfterErr)

		elif errNo == -5000:
			gProgBar.label("Error -5000: Account is locked")
			gProgBar.inc(gWaitAfterErr)
			time.sleep(1)

		elif errNo == -5023:
			gProgBar.label("Error -5023: Incorrect password")
			gProgBar.inc(gWaitAfterErr)
			time.sleep(1)

		elif (errNo == -5045) or (errNo == 32):
			gProgBar.label("Error " + str (errNo) + ": Password must be changed")
			gProgBar.inc(gWaitAfterErr)
			time.sleep(1)

		if errNo < 0:
				# Make sure that there isn't a mount point directory left behind, since we didn't succeed.
				# This should only be a problem with a negative error number.
				try: os.rmdir (volPath)
				except: pass

	return errNo

#
#	u n m o u n t V o l u m e
#

def unmountVolume():

	gProgBar.label("Unmounting network volume")
	gProgBar.inc(1)

	unmountStr = commands.getoutput ("umount " + volPathQouted)
	# Always returns 1 for Volume not mounted OR if busy
	
	# Check umount's output for problems.
	if unmountStr.find ('Device busy') >= 0:
		gProgBar.label("Error! Could not unmount volume")
		gProgBar.inc(1)
		time.sleep(gWaitAfterErr)
		sys.exit(11)
	else:
		try:
			# Make sure that there isn't a mount point directory left behind
			os.rmdir (volPath)
		except:
			pass

#
#	M a i n
#

# Do we have an active network link?
checkNetwork()

# Unmount the volume if already present (can't check password otherwise)
unmountVolume()

# Create the directory to mount the network volume in
createMount()

# Attempt to mount the volume.
errNo = mountVolume()

# If there wasn't a previous error during mount, and the user doesn't want to see the
# contents of the network volume, then unmount the volume once again.
if (not keepMounted) and (errNo == 0):
	unmountVolume()

# We are done, and also done with the progress bar
del gProgBar

# Return our error # as script's result.
sys.exit (errNo)

